#!/usr/bin/env python

### TODO
#
### Grab image for competition year out of infobox???
#
### If hyperlink in table => add in Wikipedia Song URL mapped to DBPedia
#


import json
import os
import re

import argparse

import escwikipedia

    
def generate_reverse_finishing_pos_metadata(country_year_recs):

    country_year_recs_len = len(country_year_recs)

    for country_year_key in country_year_recs.keys():
        country_year_rec = country_year_recs.get(country_year_key)

        place = int(country_year_rec.get("Place"))
        rev_place = country_year_recs_len - place + 1

        country_year_rec["ReversePlace"] = str(rev_place)
                
        country_year_rec["FinishingPos"] = str(place)
        country_year_rec["ReverseFinishingPos"] = str(rev_place)

        
def gs_fileset_country_in_year(country_year_name, country_year_rec, nul_output_dir_name):

    # Looking to build data-structure (for output as JSON) in the form
    # { "FileSet":
    #   [
    #     { "FileName": "France1991\.nul" }, 
    #     { "Description":
    #       {
    #         "Metadata":
    #           [
    #             { "name": "Place", "content": 12 }, 
    #             { "name": "Title",  "content": "...." },  
    #             ...
    #           ]
    #       }
    #     }
    #   ]
    # }

    metadata_array = []
    
    country_year_id = re.sub(r'\s+', '', country_year_name) 

    metadata_array.append({"name": "Identifier", "content": country_year_id, "mode": "override"})
        
    for metadata_key in country_year_rec.keys():
        metadata_val = country_year_rec.get(metadata_key)
        metadata_array.append({"name": metadata_key, "content": metadata_val, "mode": "override"})


    # id encodes country and year,
    filename_id = country_year_id + "\\.nul"

    nul_filename = os.path.join(nul_output_dir_name,country_year_id+".nul");

    print("  Creating: " + nul_filename)
        
    with open(nul_filename, 'w') as outfile:
        outfile.write("")
    
    fileset = {
        "FileSet" : [            
            { "FileName": filename_id, },
            { "Description" : { "Metadata" : metadata_array } }
        ]
    }
              
    return fileset



def gs_directory_metadata(all_country_year_array_of_recs, json_output_filename):

    # Express the grouped from-country voting data
    # in the Greenstone JSON metadata format:
    
    # { "DirectoryMetadata":
    #   [
    #     { "FileSet":
    #       [
    #         { "FileName": "France1991\.nul" },
    #         { "Description":
    #           {
    #             "Metadata":
    #              [
    #                { "name": "Place", "content": "12" }, 
    #                  ...
    #              ]
    #           }
    #         }
    #       ]
    #     }
    #     ...
    #    ]
    #  }

    print("**** json_output_filename = " + json_output_filename)
    
    nul_output_dir_name = os.path.dirname(json_output_filename)
    
    directory_metadata = []

    for all_in_given_year_recs in all_country_year_array_of_recs:
        
        for country_year_name in all_in_given_year_recs.keys():
            country_year_rec = all_in_given_year_recs.get(country_year_name)

            fileset = gs_fileset_country_in_year(country_year_name, country_year_rec, nul_output_dir_name)
            directory_metadata.append(fileset)
        

    greenstone_metadata_json = { "DirectoryMetadata": directory_metadata }

    return greenstone_metadata_json



def save_greenstone_json_metadata(greenstone_metadata_json,json_output_filename):

    with open(json_output_filename, 'w') as outfile:
        json.dump(greenstone_metadata_json, outfile, indent=2)
    
    
if __name__ == "__main__":
    # https://en.wikipedia.org/wiki/Eurovision_Song_Contest_1956

    # https://en.wikipedia.org/wiki/Category:Countries_in_the_Eurovision_Song_Contest_1957

    parser = argparse.ArgumentParser()
    parser.add_argument('--startyear', type=int, default=1956)
    parser.add_argument('--endyear',   type=int, default=2019)
    parser.add_argument('--cachedir', default="cache-wikipedia")
    parser.add_argument('output-file.json', nargs="?", default="metadata-esc-year/metadata-esc.json");
    
    args = parser.parse_args()

    start_year = getattr(args,'startyear')
    end_year   = getattr(args,'endyear')

    cache_dir = getattr(args,'cachedir')
    escwikipedia.init_cache(cache_dir)

    json_output_filename = getattr(args,'output-file.json')


    all_country_year_recs = []

    # final_result_ids = [ 'Grand_final', 'Final', 'Results' ]
    final_result_ids = [ 'Final', 'Results', 'Results_and_voting_controversy' ]
    
    for year in range(start_year, end_year+1):
        if (year == 2020):
            print("******")
            print("* Skipping year 2020, as no contest held that year")
            print("******")
            continue
        
        article_year_html = escwikipedia.retrieve_article_page(year)
        country_year_recs = escwikipedia.process_article_page(article_year_html,year,final_result_ids,
                                                              stop_at_first=True,
                                                              extra_metadata={"Final": True})
        if (year > 1956):
            # skip inaugural year, as only known who was 1st
            # everyone else listed as 2nd, and given that there are
            # two entrants per countries the reversing step generates
            # (while technically correct) rather weird looking vals
            generate_reverse_finishing_pos_metadata(country_year_recs)

        all_country_year_recs.append(country_year_recs)
        
    print()
    print("Generating Greenstone JSON metadata for:")
    greenstone_metadata_json = gs_directory_metadata(all_country_year_recs,json_output_filename)
    
    print()
    print("Saving output as: " + json_output_filename)    
    save_greenstone_json_metadata(greenstone_metadata_json,json_output_filename)
    print()
        
    

